/*

mtom-cr-test.c

--------------------------------------------------------------------------------
gSOAP XML Web services tools
Copyright (C) 2001-2004, Robert van Engelen, Genivia, Inc. All Rights Reserved.
This software is released under one of the following two licenses:
GPL or Genivia's license for commercial use.
--------------------------------------------------------------------------------
GPL license.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place, Suite 330, Boston, MA 02111-1307 USA

Author contact information:
engelen@genivia.com / engelen@acm.org
--------------------------------------------------------------------------------
A commercial use license is available from Genivia, Inc., contact@genivia.com
--------------------------------------------------------------------------------
*/

#include "soapH.h"
#include "mtom_cr_test.nsmap"

int cgi_serve();
int run_serve(int port);
int run_tests(int,char**);

int main(int argc, char **argv)
{ if (argc < 2)
    return cgi_serve();
  else if (argc < 3)
    return run_serve(atoi(argv[1]));
  else
    return run_tests(argc, argv);
}

int cgi_serve()
{ /* CGI-style: serve request from stdin */
  return soap_serve(soap_new1(SOAP_ENC_MTOM)); /* enable MTOM XOP attachments */
}

int run_serve(int port)
{ struct soap *soap = soap_new1(SOAP_ENC_MTOM);
  int ret;
  if (!soap_valid_socket(soap_bind(soap, NULL, port, 100)))
    soap_print_fault(soap, stderr);
  else
  { fprintf(stderr, "Bind to port %d successful\n", port);
    soap->accept_timeout = 3600; /* let server time out after one hour */
    for (;;)
    { int sock = soap_accept(soap);
      if (!soap_valid_socket(sock))
      { if (soap->errnum)
          soap_print_fault(soap, stderr);
        else
        { fprintf(stderr, "Server timed out\n");
          break;
        }
      }
      fprintf(stderr, "Accepting socket %d connection from IP %d.%d.%d.%d... ", sock, (int)(soap->ip>>24)&0xFF, (int)(soap->ip>>16)&0xFF, (int)(soap->ip>>8)&0xFF, (int)soap->ip&0xFF);
      if (soap_serve(soap))
        soap_print_fault(soap, stderr);
      fprintf(stderr, "done\n");
      soap_destroy(soap);
      soap_end(soap);
    } 
  }
  ret = soap->error;
  soap_done(soap);
  free(soap);
  return ret;
}

int run_tests(int argc, char **argv)
{ struct soap *soap = soap_new1(SOAP_ENC_MTOM);
  int i, ret;
  struct x__DataType data;
  struct x__WrapperType wrap;
  struct __m__EchoTestSingleResponse single;
  struct __m__EchoTestMultipleResponse multiple;
  soap_default_x__DataType(soap, &data);
  soap_default_x__WrapperType(soap, &wrap);
  data.__ptr = (unsigned char*)argv[2];
  data.__size = (int)strlen(argv[2]) + 1;
  if (argc > 3)
    data.xmime__contentType = argv[3];
  else
    fprintf(stderr, "No MIME type argument: using plain base64\n");
  wrap.__size = argc - 2;
  wrap.Data = (struct x__DataType*)soap_malloc(soap, wrap.__size * sizeof(struct x__DataType));
  for (i = 0; i < wrap.__size; ++i)
  { soap_default_x__DataType(soap, &wrap.Data[i]);
    wrap.Data[i].__ptr = (unsigned char*)argv[i + 2];
    wrap.Data[i].__size = (int)strlen(argv[i + 2]) + 1;
    wrap.Data[i].xmime__contentType = "text/xml";
  }
  if (soap_call___m__EchoTestSingle(soap, argv[1], NULL, &data, &single))
    soap_print_fault(soap, stderr);
  else
  { if (!single.x__Data || !single.x__Data->__ptr || single.x__Data->__size != data.__size || strcmp((char*)single.x__Data->__ptr, (char*)data.__ptr))
      fprintf(stderr, "EchoTestSingle 1: data transcription error\n");
    else
      fprintf(stderr, "EchoTestSingle 1: OK\n");
    data.type = "text/xml; charset=utf-8";
    data.xmime__contentType = NULL;
    if (soap_call___m__EchoTestSingle(soap, argv[1], NULL, &data, &single))
      soap_print_fault(soap, stderr);
    else
    { if (!single.x__Data || !single.x__Data->__ptr || single.x__Data->__size != data.__size || strcmp((char*)single.x__Data->__ptr, (char*)data.__ptr))
        fprintf(stderr, "EchoTestSingle 2: data transcription error\n");
      else
      { fprintf(stderr, "EchoTestSingle 2: OK\n");
        if (soap_call___m__EchoTestMultiple(soap, argv[1], NULL, &wrap, &multiple))
          soap_print_fault(soap, stderr);
        else
        { int okay = 1;
          if (!multiple.x__EchoTest || multiple.x__EchoTest->__size != wrap.__size)
            okay = 0;
          else
          { for (i = 0; i < multiple.x__EchoTest->__size; ++i)
              if (!multiple.x__EchoTest->Data[i].__ptr || multiple.x__EchoTest->Data[i].__size != wrap.Data[i].__size || strcmp((char*)multiple.x__EchoTest->Data[i].__ptr, (char*)wrap.Data[i].__ptr))
                okay = 0;
          }
          if (!okay)
            fprintf(stderr, "EchoTestMultiple 1: data transcription error\n");
          else
          { fprintf(stderr, "EchoTestMultiple 1: OK\n");
            for (i = 0; i < wrap.__size; ++i)
            { wrap.Data[i].type = "text/xml";
              wrap.Data[i].xmime__contentType = NULL;
            }
            if (soap_call___m__EchoTestMultiple(soap, argv[1], NULL, &wrap, &multiple))
              soap_print_fault(soap, stderr);
            else
            { int okay = 1;
              if (!multiple.x__EchoTest || multiple.x__EchoTest->__size != wrap.__size)
                okay = 0;
              else
              { for (i = 0; i < multiple.x__EchoTest->__size; ++i)
                  if (!multiple.x__EchoTest->Data[i].__ptr || multiple.x__EchoTest->Data[i].__size != wrap.Data[i].__size || strcmp((char*)multiple.x__EchoTest->Data[i].__ptr, (char*)wrap.Data[i].__ptr))
                    okay = 0;
              }
              if (!okay)
                fprintf(stderr, "EchoTestMultiple 2: data transcription error\n");
              else
                fprintf(stderr, "EchoTestMultiple 2: OK\n");
            }
          }
        }
      }
    }
  }
  ret = soap->error;
  soap_destroy(soap);
  soap_end(soap);
  soap_done(soap);
  free(soap);
  return ret;
}

int __m__EchoTestSingle(struct soap *soap, struct x__DataType *data, struct __m__EchoTestSingleResponse *response)
{ if (!data)
    return soap_sender_fault(soap, "No data", NULL);
  if (data->id)
  { /* convert MTOM XOP attachment to base64Binary with xmime:contentType */
    data->xmime__contentType = data->type;
    data->id = NULL;
    data->type = NULL;
    data->options = NULL;
  }
  else
  { /* convert base64Binary to MTOM XOP attachment */
    data->type = data->xmime__contentType;
    data->xmime__contentType = NULL;
  }
  response->x__Data = data;
  return SOAP_OK;
}

int __m__EchoTestMultiple(struct soap *soap, struct x__WrapperType *x__EchoTest, struct __m__EchoTestMultipleResponse *response)
{ int i;
  if (!x__EchoTest)
    return soap_sender_fault(soap, "No data", NULL);
  for (i = 0; i < x__EchoTest->__size; ++i)
  { if (x__EchoTest->Data[i].id)
    { /* convert MTOM XOP attachment to base64Binary with xmime:contentType */
      x__EchoTest->Data[i].xmime__contentType = x__EchoTest->Data[i].type;
      x__EchoTest->Data[i].id = NULL;
      x__EchoTest->Data[i].type = NULL;
      x__EchoTest->Data[i].options = NULL;
    }
    else
    { /* convert base64Binary to MTOM XOP attachment */
      x__EchoTest->Data[i].type = x__EchoTest->Data[i].xmime__contentType;
      x__EchoTest->Data[i].xmime__contentType = NULL;
    }
  }
  response->x__EchoTest = x__EchoTest;
  return SOAP_OK;
}
